@@ -10,7 +10,7 @@ extern crate regex;
 use chrono::Utc;
 use regex::Regex;
 
-#[derive(Debug)]
+#[derive(Debug, PartialEq)]
 pub struct Include {
     pub captured: String,
 }
@@ -86,9 +86,10 @@ impl Parser {
 ///     - **@callee.name@** : the function name of callee
 ///     - **@callee.rtype@** : the return type of callee
 ///     - **@callee.rtype.change(\<from\>=\<to\>)@** : to change return data type during generation
-///     - **@callee.rtype.variable(\<var\>)@** : \<var\> string to be removed when `void`
-///     - **@callee.rtype.value(\<val\>)@** : \<val\> string to be removed when `void`
+///     - **@callee.rtype.remove(\<text\>)@** : \<text\> to be removed when `void`
+///     - **@callee.rtype.remove0(\<text\>)@** : \<text\> to be removed when `void`
 ///     - **@callee.args@** : the argument list string
+///     - **@callee.args.remove(\<text\>)@** : \<text\> to be removed when `void`
 ///     - **@callee.atypes@** : only arguments' data types
 ///     - **@caller.name@** : the function name of caller
 ///     - **@caller.rtype@** : the return type of caller
@@ -240,25 +241,30 @@ pub fn generate<'a>(parser: &'a Parser, template: &'a str, sourcename: &'a str)
                         fmtstr = re4change.replace(&fmtstr, &entry.callee.rtype).into_owned();
                     }
                 }
-                let re4variable =
-                    Regex::new(r"@callee.rtype.variable\((?P<variable>[a-z|A-Z|0-9|_]+)\)@")
-                        .unwrap();
-                for cap in re4variable.captures_iter(fmtstr.clone().as_str()) {
-                    if entry.callee.rtype.as_str() == "void" {
-                        fmtstr = re4variable.replace(&fmtstr, "").into_owned();
-                    } else {
-                        let variable = cap.name("variable").unwrap().as_str();
-                        fmtstr = re4variable.replace(&fmtstr, variable).into_owned();
+                // remove tags for callee.rtype
+                let remove_tags = vec!["callee.rtype.remove", "callee.rtype.remove0"];
+                for tag in remove_tags {
+                    let regstr = format!(r"@{}\((?P<text>[^)]+)\)@", tag);
+                    let re = Regex::new(&regstr).unwrap();
+                    for cap in re.captures_iter(fmtstr.clone().as_str()) {
+                        if entry.callee.rtype.as_str() == "void" {
+                            fmtstr = re.replace(&fmtstr, "").into_owned();
+                        } else {
+                            let text = cap.name("text").unwrap().as_str();
+                            fmtstr = re.replace(&fmtstr, text).into_owned();
+                        }
                     }
                 }
-                let re4value =
-                    Regex::new(r"@callee.rtype.value\((?P<value>[a-z|A-Z|0-9|_]+)\)@").unwrap();
-                for cap in re4value.captures_iter(fmtstr.clone().as_str()) {
-                    if entry.callee.rtype.as_str() == "void" {
-                        fmtstr = re4value.replace(&fmtstr, "").into_owned();
+                // remove tag for callee.args
+                let remove_tag = "callee.args.remove";
+                let regstr = format!(r"@{}\((?P<text>[^)]+)\)@", remove_tag);
+                let re = Regex::new(&regstr).unwrap();
+                for cap in re.captures_iter(fmtstr.clone().as_str()) {
+                    if entry.callee.args.as_str() == "void" || entry.callee.args.as_str() == "" {
+                        fmtstr = re.replace(&fmtstr, "").into_owned();
                     } else {
-                        let value = cap.name("value").unwrap().as_str();
-                        fmtstr = re4value.replace(&fmtstr, value).into_owned();
+                        let text = cap.name("text").unwrap().as_str();
+                        fmtstr = re.replace(&fmtstr, text).into_owned();
                     }
                 }
                 tmpstr.push_str(&fmtstr);
@@ -289,6 +295,7 @@ fn get_incs<'a>(code: &'a str) -> Vec<Include> {
             captured: cap.name("captured").unwrap().as_str().trim().to_string(),
         });
     }
+    result.dedup();
     result
 }
 
@@ -366,10 +373,18 @@ fn get_fncs<'a>(code: &'a str) -> Vec<Function> {
         type_list
     };
     for cap in re.captures_iter(code) {
+        if cap.name("name").unwrap().as_str().trim() == "if" {
+            continue;
+        }
         let re4space = Regex::new(r"\s+").unwrap();
-        let raw_args = re4space
+        let mut raw_args = re4space
             .replace_all(cap.name("args").unwrap().as_str().trim(), " ")
+            .replace("\\", "")
+            .trim()
             .to_string();
+        if raw_args.trim() == "void" {
+            raw_args.clear();
+        }
         result.push(Function {
             captured: cap.get(0).unwrap().as_str().trim().to_string(),
             name: cap.name("name").unwrap().as_str().trim().to_string(),
@@ -438,9 +453,11 @@ fn get_ncls<'a>(code: &'a str, fncs: &Vec<Function>) -> Vec<NestedCall> {
 #[cfg(test)]
 mod tests {
     use super::*;
+    use std::fs;
 
     static TEST_CODE: &'static str = "\
 #include <stdio.h>
+#include <stdio.h>
 #include \"test.h\"
 
 int global_var = 2;
@@ -474,6 +491,13 @@ static inline char local_function(int a,
 }
 ";
 
+    #[test]
+    fn test_parse() {
+        let code = fs::read_to_string("sample.c").unwrap();
+        let parser = Parser::parse(&code);
+        println!("{:#?}", parser);
+    }
+
     #[test]
     fn test_remove_comments() {
         let clean_code = remove_comments(TEST_CODE);
@@ -613,12 +637,14 @@ int func2(int a)
 ";
         let temp = "\
 @ncls@- @caller.name@ -> @callee.name@
-    - return @callee.rtype.value(0)@;
+    - return @callee.rtype.remove(0)@;
+    - (int dummy@callee.args.remove(, )@@callee.args@);
 @end-ncls@
 ";
         let expected = "\
 - func2 -> func1
     - return ;
+    - (int dummy);
 
 ";
         let parser = Parser::parse(code);
