@@ -98,7 +98,7 @@ int func2(char c)
 int var;
 
 // functions
-int func1(void);
+int func1();
 int func2(char c);
 
 ";
